@@ -1,11 +1,30 @@
-///////////////////////////////////////////////////////////////////////////////////////////////////
-// OpenGL Mathematics Copyright (c) 2005 - 2011 G-Truc Creation (www.g-truc.net)
-///////////////////////////////////////////////////////////////////////////////////////////////////
-// Created : 2006-11-13
-// Updated : 2011-01-26
-// Licence : This source is under MIT License
-// File    : glm/setup.hpp
-///////////////////////////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////////
+/// OpenGL Mathematics (glm.g-truc.net)
+///
+/// Copyright (c) 2005 - 2012 G-Truc Creation (www.g-truc.net)
+/// Permission is hereby granted, free of charge, to any person obtaining a copy
+/// of this software and associated documentation files (the "Software"), to deal
+/// in the Software without restriction, including without limitation the rights
+/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+/// copies of the Software, and to permit persons to whom the Software is
+/// furnished to do so, subject to the following conditions:
+/// 
+/// The above copyright notice and this permission notice shall be included in
+/// all copies or substantial portions of the Software.
+/// 
+/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+/// THE SOFTWARE.
+///
+/// @ref core
+/// @file glm/core/setup.hpp
+/// @date 2006-11-13 / 2011-06-15
+/// @author Christophe Riccio
+///////////////////////////////////////////////////////////////////////////////////
 
 #ifndef glm_setup
 #define glm_setup
@@ -13,11 +32,65 @@
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 // Version
 
-#define GLM_VERSION					92
+#define GLM_VERSION					93
 #define GLM_VERSION_MAJOR			0
 #define GLM_VERSION_MINOR			9
-#define GLM_VERSION_PATCH			2
-#define GLM_VERSION_REVISION		6
+#define GLM_VERSION_PATCH			3
+#define GLM_VERSION_REVISION		1
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+// Platform
+
+#define GLM_PLATFORM_UNKNOWN		0x00000000
+#define GLM_PLATFORM_WINDOWS		0x00010000
+#define GLM_PLATFORM_LINUX			0x00020000
+#define GLM_PLATFORM_APPLE			0x00040000
+//#define GLM_PLATFORM_IOS			0x00080000
+#define GLM_PLATFORM_ANDROID		0x00100000
+#define GLM_PLATFORM_CHROME_NACL	0x00200000
+#define GLM_PLATFORM_UNIX			0x00400000
+
+#ifdef GLM_FORCE_PLATFORM_UNKNOWN
+#	define GLM_PLATFORM GLM_PLATFORM_UNKNOWN
+#elif defined(__APPLE__)
+#   define GLM_PLATFORM GLM_PLATFORM_APPLE
+#elif defined(_WIN32)
+#	define GLM_PLATFORM GLM_PLATFORM_WINDOWS
+#elif defined(__native_client__)
+#	define GLM_PLATFORM GLM_PLATFORM_CHROME_NACL
+#elif defined(ANDROID)
+#   define GLM_PLATFORM GLM_PLATFORM_ANDROID
+#elif defined(__linux)
+#   define GLM_PLATFORM GLM_PLATFORM_LINUX
+#elif defined(__unix)
+#   define GLM_PLATFORM GLM_PLATFORM_UNIX
+#else
+#	define GLM_PLATFORM GLM_PLATFORM_UNKNOWN
+#endif//
+
+// Report platform detection
+#if(defined(GLM_MESSAGES) && !defined(GLM_MESSAGE_PLATFORM_DISPLAYED))
+#	define GLM_MESSAGE_PLATFORM_DISPLAYED
+#	if(GLM_PLATFORM & GLM_PLATFORM_WINDOWS)
+#		pragma message("GLM: Windows platform detected")
+//#	elif(GLM_PLATFORM & GLM_PLATFORM_IOS)
+//#		pragma message("GLM: iOS platform detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_APPLE)
+#		pragma message("GLM: Apple platform detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_LINUX)
+#		pragma message("GLM: Linux platform detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_UNIX)
+#		pragma message("GLM: UNIX platform detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_ANDROID)
+#		pragma message("GLM: Android platform detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_CHROME_NACL)
+#		pragma message("GLM: Chrone Native Client detected")
+#	elif(GLM_PLATFORM & GLM_PLATFORM_UNKNOWN)
+#		pragma message("GLM: platform unknown")
+#	else
+#		pragma message("GLM: platform not detected")
+#	endif
+#endif//GLM_MESSAGE
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 // Compiler
@@ -82,6 +155,8 @@
 #define GLM_COMPILER_CUDA31			0x10000020
 #define GLM_COMPILER_CUDA32			0x10000030
 #define GLM_COMPILER_CUDA40			0x10000040
+#define GLM_COMPILER_CUDA41			0x10000050
+#define GLM_COMPILER_CUDA42			0x10000060
 
 // Clang
 #define GLM_COMPILER_CLANG          0x20000000
@@ -89,21 +164,76 @@
 #define GLM_COMPILER_CLANG27		0x20000020
 #define GLM_COMPILER_CLANG28		0x20000030
 #define GLM_COMPILER_CLANG29		0x20000040
+#define GLM_COMPILER_CLANG30		0x20000050
+#define GLM_COMPILER_CLANG31		0x20000060
+#define GLM_COMPILER_CLANG32		0x20000070
+#define GLM_COMPILER_CLANG33		0x20000080
 
 // LLVM GCC
 #define GLM_COMPILER_LLVM_GCC		0x40000000
 
+// Intel
+#define GLM_COMPILER_INTEL          0x80000000
+#define GLM_COMPILER_INTEL9			0x80000010
+#define GLM_COMPILER_INTEL10_0		0x80000020
+#define GLM_COMPILER_INTEL10_1		0x80000030
+#define GLM_COMPILER_INTEL11_0		0x80000040
+#define GLM_COMPILER_INTEL11_1		0x80000050
+#define GLM_COMPILER_INTEL12_0		0x80000060
+#define GLM_COMPILER_INTEL12_1		0x80000070
+#define GLM_COMPILER_INTEL13_0		0x80000080
+
 // Build model
 #define GLM_MODEL_32				0x00000010
 #define GLM_MODEL_64				0x00000020
 
 // Force generic C++ compiler
 #ifdef GLM_FORCE_COMPILER_UNKNOWN
-#		define GLM_COMPILER GLM_COMPILER_UNKNOWN
+#	define GLM_COMPILER GLM_COMPILER_UNKNOWN
+
+#elif defined(__INTEL_COMPILER)
+#	if __INTEL_COMPILER == 900
+#		define GLM_COMPILER GLM_COMPILER_INTEL9
+#	elif __INTEL_COMPILER == 1000
+#		define GLM_COMPILER GLM_COMPILER_INTEL10_0
+#	elif __INTEL_COMPILER == 1010
+#		define GLM_COMPILER GLM_COMPILER_INTEL10_1
+#	elif __INTEL_COMPILER == 1100
+#		define GLM_COMPILER GLM_COMPILER_INTEL11_0
+#	elif __INTEL_COMPILER == 1110
+#		define GLM_COMPILER GLM_COMPILER_INTEL11_1
+#	elif __INTEL_COMPILER == 1200
+#		define GLM_COMPILER GLM_COMPILER_INTEL12_0
+#	elif __INTEL_COMPILER == 1210
+#		define GLM_COMPILER GLM_COMPILER_INTEL12_1
+#	elif __INTEL_COMPILER == 1300
+#		define GLM_COMPILER GLM_COMPILER_INTEL13_0
+#	else
+#		define GLM_COMPILER GLM_COMPILER_INTEL
+#	endif
 
 // CUDA
 #elif defined(__CUDACC__)
 #	define GLM_COMPILER GLM_COMPILER_CUDA
+/*
+#	if CUDA_VERSION < 3000
+#		error "GLM requires CUDA 3.0 or higher"
+#	elif CUDA_VERSION == 3000
+#		define GLM_COMPILER GLM_COMPILER_CUDA30	
+#	elif CUDA_VERSION == 3010
+#		define GLM_COMPILER GLM_COMPILER_CUDA31	
+#	elif CUDA_VERSION == 3020
+#		define GLM_COMPILER GLM_COMPILER_CUDA32	
+#	elif CUDA_VERSION == 4000
+#		define GLM_COMPILER GLM_COMPILER_CUDA40	
+#	elif CUDA_VERSION == 4010
+#		define GLM_COMPILER GLM_COMPILER_CUDA41	
+#	elif CUDA_VERSION == 4020
+#		define GLM_COMPILER GLM_COMPILER_CUDA42
+#	else
+#		define GLM_COMPILER GLM_COMPILER_CUDA
+#	endif
+*/
 
 // Visual C++
 #elif defined(_MSC_VER)
@@ -131,8 +261,30 @@
 #		define GLM_COMPILER GLM_COMPILER_VC
 #	endif//_MSC_VER
 
-// G++
-#elif defined(__GNUC__) || defined(__llvm__) || defined(__clang__)
+// Clang
+#elif defined(__clang__)
+#	if(CLANG_VERSION_MAJOR == 2) && (CLANG_VERSION_MINOR == 6)
+#		define GLM_COMPILER GLM_COMPILER_CLANG26
+#	elif(CLANG_VERSION_MAJOR == 2) && (CLANG_VERSION_MINOR == 7)
+#		define GLM_COMPILER GLM_COMPILER_CLANG27
+#	elif(CLANG_VERSION_MAJOR == 2) && (CLANG_VERSION_MINOR == 8)
+#		define GLM_COMPILER GLM_COMPILER_CLANG28
+#	elif(CLANG_VERSION_MAJOR == 2) && (CLANG_VERSION_MINOR == 9)
+#		define GLM_COMPILER GLM_COMPILER_CLANG29
+#	elif(CLANG_VERSION_MAJOR == 3) && (CLANG_VERSION_MINOR == 0)
+#		define GLM_COMPILER GLM_COMPILER_CLANG30
+#	elif(CLANG_VERSION_MAJOR == 3) && (CLANG_VERSION_MINOR == 1)
+#		define GLM_COMPILER GLM_COMPILER_CLANG31
+#	elif(CLANG_VERSION_MAJOR == 3) && (CLANG_VERSION_MINOR == 2)
+#		define GLM_COMPILER GLM_COMPILER_CLANG32
+#	elif(CLANG_VERSION_MAJOR == 3) && (CLANG_VERSION_MINOR == 3)
+#		define GLM_COMPILER GLM_COMPILER_CLANG33
+#	else
+#		define GLM_COMPILER GLM_COMPILER_CLANG
+#   endif
+
+// G++ 
+#elif(defined(__GNUC__) || defined(__MINGW32__))// || defined(__llvm__) || defined(__clang__)
 #   if defined (__llvm__)
 #       define GLM_COMPILER_GCC_EXTRA GLM_COMPILER_GCC_LLVM
 #   elif defined (__clang__)
@@ -232,18 +384,10 @@
 /////////////////
 // Build model //
 
-#if(GLM_COMPILER & GLM_COMPILER_VC)
-#	if defined(_M_X64)
+#if((defined(__WORDSIZE) && (__WORDSIZE == 64)) || defined(__arch64__) || defined(__LP64__) || defined(_M_X64) || defined(__ppc64__) || defined(__x86_64__))
 #		define GLM_MODEL	GLM_MODEL_64
-#	else
-#		define GLM_MODEL	GLM_MODEL_32
-#	endif//_M_X64
-#elif(GLM_COMPILER & GLM_COMPILER_GCC)
-#	if(defined(__WORDSIZE) && (__WORDSIZE == 64)) || defined(__arch64__) || defined(__LP64__) || defined(__x86_64__)
-#		define GLM_MODEL	GLM_MODEL_64
-#	else
-#		define GLM_MODEL	GLM_MODEL_32
-#	endif//
+#elif(defined(__i386__) || defined(__ppc__))
+#	define GLM_MODEL	GLM_MODEL_32
 #else
 #	define GLM_MODEL	GLM_MODEL_32
 #endif//
@@ -266,32 +410,57 @@
 
 // User defines: GLM_FORCE_CXX98
 
-#define GLM_LANG_CXX			0
-#define GLM_LANG_CXX98			1
-#define GLM_LANG_CXX0X			2
-#define GLM_LANG_CXXMS			3
-#define GLM_LANG_CXXGNU			4
+#define GLM_LANG_CXX			(0 << 0)
+#define GLM_LANG_CXX98			((1 << 1) | GLM_LANG_CXX)
+#define GLM_LANG_CXX03			((1 << 2) | GLM_LANG_CXX98)
+#define GLM_LANG_CXX0X			((1 << 3) | GLM_LANG_CXX03)
+#define GLM_LANG_CXX11			((1 << 4) | GLM_LANG_CXX0X)
+#define GLM_LANG_CXXMS			(1 << 5)
+#define GLM_LANG_CXXGNU			(1 << 6)
 
-#if(defined(GLM_FORCE_CXX98))
-#	define GLM_LANG GLM_LANG_CXX98
-#elif(((GLM_COMPILER & GLM_COMPILER_GCC) == GLM_COMPILER_GCC) && defined(__GXX_EXPERIMENTAL_CXX0X__)) // -std=c++0x or -std=gnu++0x
-#	define GLM_LANG GLM_LANG_CXX0X
-#elif(GLM_COMPILER == GLM_COMPILER_VC2010) //_MSC_EXTENSIONS for MS language extensions
-#	define GLM_LANG GLM_LANG_CXX0X
-#elif(((GLM_COMPILER & GLM_COMPILER_GCC) == GLM_COMPILER_GCC) && defined(__STRICT_ANSI__))
-#	define GLM_LANG GLM_LANG_CXX98
-#elif(((GLM_COMPILER & GLM_COMPILER_VC) == GLM_COMPILER_VC) && !defined(_MSC_EXTENSIONS))
+#if(defined(GLM_FORCE_CXX11))
+#	define GLM_LANG GLM_LANG_CXX11
+#elif(defined(GLM_FORCE_CXX03))
+#	define GLM_LANG GLM_LANG_CXX03
+#elif(defined(GLM_FORCE_CXX98))
 #	define GLM_LANG GLM_LANG_CXX98
 #else
-#	define GLM_LANG GLM_LANG_CXX
+//  -std=c++0x or -std=gnu++0x
+#	if(((GLM_COMPILER & GLM_COMPILER_GCC) == GLM_COMPILER_GCC) && defined(__GXX_EXPERIMENTAL_CXX0X__)) 
+#		define GLM_LANG GLM_LANG_CXX0X
+#	elif(((GLM_COMPILER & GLM_COMPILER_VC) == GLM_COMPILER_VC) && defined(_MSC_EXTENSIONS))
+#		define GLM_LANG GLM_LANG_CXXMS
+#	elif(((GLM_COMPILER & GLM_COMPILER_VC) == GLM_COMPILER_VC) && !defined(_MSC_EXTENSIONS))
+#		if(GLM_COMPILER >= GLM_COMPILER_VC2010)
+#			define GLM_LANG GLM_LANG_CXX0X
+#		else
+#			define GLM_LANG GLM_LANG_CXX98
+#		endif//(GLM_COMPILER == GLM_COMPILER_VC2010)
+#	elif((GLM_COMPILER & GLM_COMPILER_GCC) == GLM_COMPILER_GCC) //&& defined(__STRICT_ANSI__))
+#		define GLM_LANG GLM_LANG_CXX98
+#	elif((GLM_COMPILER & GLM_COMPILER_CLANG) == GLM_COMPILER_CLANG) 
+#		define GLM_LANG GLM_LANG_CXX98
+#	else
+#		define GLM_LANG GLM_LANG_CXX
+#	endif
 #endif
 
 #if(defined(GLM_MESSAGES) && !defined(GLM_MESSAGE_LANG_DISPLAYED))
 #	define GLM_MESSAGE_LANG_DISPLAYED
 #	if(GLM_LANG == GLM_LANG_CXX98)
 #		pragma message("GLM: C++98")
+#	elif(GLM_LANG == GLM_LANG_CXX03)
+#		pragma message("GLM: C++03")
 #	elif(GLM_LANG == GLM_LANG_CXX0X)
 #		pragma message("GLM: C++0x")
+#	elif(GLM_LANG == GLM_LANG_CXX11)
+#		pragma message("GLM: C++11")
+#	elif(GLM_LANG == GLM_LANG_CXXGNU)
+#		pragma message("GLM: C++ with GNU language extensions")
+#	elif(GLM_LANG == GLM_LANG_CXXMS)
+#		pragma message("GLM: C++ with VC language extensions")
+#   else
+#       pragma message("GLM: C++ language undetected")
 #	endif//GLM_MODEL
 #endif//GLM_MESSAGE
 
@@ -370,46 +539,54 @@
 #	if(GLM_ARCH == GLM_ARCH_PURE)
 #		pragma message("GLM: Platform independent")
 #	elif(GLM_ARCH == GLM_ARCH_SSE2)
-#		pragma message("GLM: SSE2 build platform")
+#		pragma message("GLM: SSE2 instruction set")
 #	elif(GLM_ARCH == GLM_ARCH_SSE3)
-#		pragma message("GLM: SSE3 build platform")
+#		pragma message("GLM: SSE3 instruction set")
 #	elif(GLM_ARCH == GLM_ARCH_AVX)
-#		pragma message("GLM: AVX build platform")
+#		pragma message("GLM: AVX instruction set")
 #	endif//GLM_ARCH
 #endif//GLM_MESSAGE
 
+///////////////////////////////////////////////////////////////////////////////////////////////////
+// Support check macros
+
+#define GLM_SUPPORT_ANONYMOUS_UNION() \
+	((GLM_LANG & GLM_LANG_CXX98) == GLM_LANG_CXX98)
+
+#define GLM_SUPPORT_ANONYMOUS_UNION_OF_STRUCTURE() \
+	(((GLM_LANG & GLM_LANG_CXX11) == GLM_LANG_CXX11) || ((GLM_LANG & GLM_LANG_CXXMS) == GLM_LANG_CXXMS) || ((GLM_COMPILER & GLM_COMPILER_GCC) && (GLM_LANG == GLM_LANG_CXX0X)))
+
+#define GLM_SUPPORT_SWIZZLE_OPERATOR() \
+	(/*defined(GLM_SWIZZLE) && */GLM_SUPPORT_ANONYMOUS_UNION_OF_STRUCTURE())
+
+#define GLM_SUPPORT_SWIZZLE_FUNCTION() defined(GLM_SWIZZLE)
+
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 // Components
 
 //#define GLM_FORCE_ONLY_XYZW
-#define GLM_COMPONENT_GLSL_NAMES			0 
-#define GLM_COMPONENT_ONLY_XYZW				1 // To disable multiple vector component names access.
-#define GLM_COMPONENT_MS_EXT				2 // To use anonymous union to provide multiple component names access for class valType. Visual C++ only.
+#define GLM_COMPONENT_ONLY_XYZW				0 // To disable multiple vector component names access.
+#define GLM_COMPONENT_CXX98					1 //  
+#define GLM_COMPONENT_CXX11					2 // To use anonymous union to provide multiple component names access for class valType. Visual C++ only.
 
-#ifndef GLM_FORCE_ONLY_XYZW
-#	if((GLM_COMPILER & GLM_COMPILER_VC) && defined(_MSC_EXTENSIONS))
-#		define GLM_COMPONENT GLM_COMPONENT_MS_EXT
-#	else
-#		define GLM_COMPONENT GLM_COMPONENT_GLSL_NAMES
-#	endif
+#if(GLM_SUPPORT_ANONYMOUS_UNION_OF_STRUCTURE() && !defined(GLM_FORCE_ONLY_XYZW))
+#	define GLM_COMPONENT GLM_COMPONENT_CXX11
+#elif(GLM_SUPPORT_ANONYMOUS_UNION() && !defined(GLM_FORCE_ONLY_XYZW))
+#	define GLM_COMPONENT GLM_COMPONENT_CXX98
 #else
 #	define GLM_COMPONENT GLM_COMPONENT_ONLY_XYZW
 #endif
 
-#if((GLM_COMPONENT == GLM_COMPONENT_MS_EXT) && !(GLM_COMPILER & GLM_COMPILER_VC))
-#	error "GLM_COMPONENT value is GLM_COMPONENT_MS_EXT but this is not allowed with the current compiler."
-#endif
-
 #if(defined(GLM_MESSAGES) && !defined(GLM_MESSAGE_COMPONENT_DISPLAYED))
 #	define GLM_MESSAGE_COMPONENT_DISPLAYED
-#	if(GLM_COMPONENT == GLM_COMPONENT_GLSL_NAMES)
-#		pragma message("GLM: GLSL multiple vector component names")
+#	if(GLM_COMPONENT == GLM_COMPONENT_CXX98)
+#		pragma message("GLM: x,y,z,w; r,g,b,a; s,t,p,q component names except of half based vector types")
 #	elif(GLM_COMPONENT == GLM_COMPONENT_ONLY_XYZW)
-#		pragma message("GLM: x,y,z,w vector component names only")
-#	elif(GLM_COMPONENT == GLM_COMPONENT_MS_EXT)
-#		pragma message("GLM: Multiple vector component names through Visual C++ language extensions")
+#		pragma message("GLM: x,y,z,w component names for all vector types")
+#	elif(GLM_COMPONENT == GLM_COMPONENT_CXX11)
+#		pragma message("GLM: x,y,z,w; r,g,b,a; s,t,p,q component names for all vector types")
 #	else
-#		error "GLM_COMPONENT value unknown"
+#		error "GLM: GLM_COMPONENT value unknown"
 #	endif//GLM_MESSAGE_COMPONENT_DISPLAYED
 #endif//GLM_MESSAGE
 
@@ -464,16 +641,16 @@
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 // Swizzle operators
 
-// User defines: GLM_SWIZZLE_XYZW GLM_SWIZZLE_RGBA GLM_SWIZZLE_STQP GLM_SWIZZLE
+// User defines: GLM_SWIZZLE
 
 #if(defined(GLM_MESSAGES) && !defined(GLM_MESSAGE_SWIZZLE_DISPLAYED))
 #	define GLM_MESSAGE_SWIZZLE_DISPLAYED
-#	if(defined(GLM_SWIZZLE))
-#		pragma message("GLM: Full swizzling operator enabled")
-#	elif(!defined(GLM_SWIZZLE_XYZW) && !defined(GLM_SWIZZLE_RGBA) && !defined(GLM_SWIZZLE_STQP) && !defined(GLM_SWIZZLE))
-#		pragma message("GLM: No swizzling operator enabled")
+#	if(GLM_SUPPORT_SWIZZLE_OPERATOR())
+#		pragma message("GLM: Swizzling operators enabled")
+#	elif(GLM_SUPPORT_SWIZZLE_FUNCTION())
+#		pragma message("GLM: Swizzling operators supported through swizzling functions")
 #	else
-#		pragma message("GLM: Partial swizzling operator enabled")
+#		pragma message("GLM: Swizzling operators disabled")
 #	endif
 #endif//GLM_MESSAGE
 
